# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# Time       ：2024/4/22 18:36
# Author     ：Maxwell
# Description：
"""
import time
from loguru import logger
from typing import List, Callable
from functools import wraps
from inspect import iscoroutinefunction


logger.add("./logs/info.log", level='INFO', rotation="200 MB")
logger.add("./logs/warning.log", level='WARNING', rotation="200 MB")
logger.add("./logs/debug.log", level='DEBUG', rotation="200 MB")
logger.add("./logs/error.log", level='ERROR', rotation="200 MB")


def error_handler(func: Callable):
    @wraps(func)
    async def async_wrapper(*args, **kwargs):
        try:
            return await func(*args, **kwargs)
        except Exception as e:
            logger.error(f"Error in {func.__name__}: {e}")
            return None

    @wraps(func)
    def sync_wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            logger.error(f"Error in {func.__name__}: {e}")
            return None

    if iscoroutinefunction(func):
        return async_wrapper
    else:
        return sync_wrapper


def measure_time(mark="耗时"):
    def decorator(func):
        @wraps(func)
        async def async_wrapper(*args, **kwargs):
            start_time = time.perf_counter()
            result = await func(*args, **kwargs)
            end_time = time.perf_counter()
            logger.info(f"{mark}: {func.__name__}, {end_time - start_time:.4f} s")
            return result

        @wraps(func)
        def sync_wrapper(*args, **kwargs):
            start_time = time.perf_counter()
            result = func(*args, **kwargs)
            end_time = time.perf_counter()
            logger.info(f"{mark}: {func.__name__}, {end_time - start_time:.4f} s")
            return result

        if iscoroutinefunction(func):
            return async_wrapper
        else:
            return sync_wrapper
    return decorator


# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# Time       ：2025/1/6 15:46
# Author     ：Maxwell
# Description：
"""

import hashlib
import uuid
import time
import random


class HashManager(object):

    @classmethod
    def md5(cls, content: str) -> str:
        if not content:
            content = ""
        md5_hash = hashlib.md5()
        md5_hash.update(content.encode('utf-8'))
        return md5_hash.hexdigest()

    @classmethod
    def sha256(cls, content: str) -> str:
        if not content:
            content = ""
        sha256_hash = hashlib.sha256()
        sha256_hash.update(content.encode('utf-8'))
        return sha256_hash.hexdigest().upper()

    @classmethod
    def generate_uuid(cls) -> str:
        return uuid.uuid4().hex.upper()

    @classmethod
    def uuid(cls) -> str:
        uuid_str = uuid.uuid1().hex.upper()
        unique_id = "-".join([str(random.randint(0, 100_0000_0000)) for _ in range(10)])
        return cls.sha256(f"{uuid_str}-{int(time.time())}-{unique_id}").upper()

    @classmethod
    def is_sha256(cls, string):
        if len(string) != 64:
            return False
        try:
            hashlib.sha256(string.encode()).hexdigest()
            return True
        except ValueError:
            return False





import paho.mqtt.client as mqtt
import asyncio
from typing import Dict, Optional, Callable, Any, List
from hwhkit.utils import logger
from hwhkit.connection.mqtt.plugins import PluginBase
from hwhkit.connection.mqtt.plugins.encryption import EncryptionPlugin


class MQTTClientManager:
    def __init__(self, add_plugins=True, mqtt_config="mqtt_key_pairs.yaml"):
        self.clients: Dict[str, mqtt.Client] = {}
        self.topic_callbacks: Dict[str, List[Callable[[mqtt.Client, str], Any]]] = {}
        self.plugins = []

        if add_plugins:
            self.plugins.append(EncryptionPlugin(mqtt_config))
            # self.plugins.append(ProtobufPlugin())
            self.reversed_plugins = list(reversed(self.plugins))

    def create_client(
        self,
        client_id: str,
        broker: str,
        port: int = 1883,
        username: Optional[str] = None,
        password: Optional[str] = None,
    ) -> 'MQTTClientManager':
        if client_id in self.clients:
            raise ValueError(f"Client with ID {client_id} already exists.")

        client = mqtt.Client(client_id=client_id)
        if username and password:
            client.username_pw_set(username, password)

        client.broker = broker
        client.port = port
        client.on_connect = self._on_connect
        client.on_disconnect = self._on_disconnect
        client.reconnect_delay_set(min_delay=1, max_delay=120)

        self.clients[client_id] = client
        return self

    def get_client(self, client_id: str) -> Optional[mqtt.Client]:
        return self.clients.get(client_id)

    def remove_client(self, client_id: str):
        client = self.clients.pop(client_id, None)
        if client:
            client.disconnect()

    def add_plugin(self, plugin: PluginBase) -> 'MQTTClientManager':
        self.plugins.append(plugin)
        return self

    def start_all_clients(self) -> 'MQTTClientManager':
        for client_id, client in self.clients.items():
            client.connect(client.broker, client.port, 60)
            client.loop_start()
        return self

    def start_client(self, client_id: str):
        client = self.get_client(client_id)
        if client:
            client.connect(client.broker, client.port, 60)
            client.loop_start()
        else:
            raise ValueError(f"Client with ID {client_id} not found.")

    def _on_connect(self, client: mqtt.Client, userdata, flags, rc):
        if rc == 0:
            logger.info(f"Client {client._client_id.decode()} connected to MQTT Broker!")
        else:
            logger.info(f"Client {client._client_id.decode()} failed to connect, return code {rc}")

    def _on_disconnect(self, client: mqtt.Client, userdata, rc):
        logger.info(f"Client {client._client_id.decode()} disconnected from MQTT Broker!")
        if rc != 0:
            logger.info(f"Unexpected disconnection. Reconnecting...")
            client.reconnect()

    def _on_message(self, topic: str, client: mqtt.Client, msg: mqtt.MQTTMessage):
        logger.info(f"Received topic@{topic}, {msg.payload}")
        try:
            if topic in self.topic_callbacks:
                processed_message = msg.payload
                for plugin in self.reversed_plugins:
                    processed_message = plugin.on_message_received(topic, processed_message)
                for callback in self.topic_callbacks[topic]:
                    callback(client, processed_message)
        except Exception as e:
            logger.error(f"on_message error: {e}")

    def subscribe(self, topic: str):
        def decorator(callback: Callable[[mqtt.Client, str], Any]):
            if topic not in self.topic_callbacks:
                self.topic_callbacks[topic] = []

            self.topic_callbacks[topic].append(callback)

            for client_id, client in self.clients.items():
                client.subscribe(topic)
                client.message_callback_add(
                    topic,
                    lambda c, userdata, msg: self._on_message(topic, c, msg),
                )
            logger.info(f"Subscribed to topic: {topic}")
            return callback

        return decorator

    def publish(self, client_id: str, topic: str, message: str, qos: int = 0, retain: bool = False):
        client = self.get_client(client_id)
        if client:
            if not client.is_connected():
                logger.info(f"Client {client_id} is not connected. Attempting to reconnect...")
                client.reconnect()

            processed_message = message
            for plugin in self.plugins:
                processed_message = plugin.on_message_published(topic, processed_message)

            result = client.publish(topic, processed_message, qos=qos, retain=retain)
            if result.rc == mqtt.MQTT_ERR_SUCCESS:
                logger.info(f"Client {client_id} published message to {topic}")
            else:
                logger.info(f"Client {client_id} failed to publish message to {topic}, error code: {result.rc}")
        else:
            raise ValueError(f"Client with ID {client_id} not found.")


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='hwhkit',
    version='1.0.16',
    description='Packaging tools for own use',
    author='louishwh',
    author_email='louishwh@gmail.com',
    url='',
    #packages=['hwhkit.connection'],
    packages=setuptools.find_packages(),
    long_description=long_description,
    long_description_content_type="text/markdown",
    python_requires='>=3.10',
    install_requires=[
        'aiomqtt>=2.3.0',
        'aiohttp>=3.11.10',
        'aiofiles~=24.1.0',
        'loguru==0.6.0',
        'openai~=1.58.1',
        'anthropic~=0.42.0',
        'boto3~=1.34.79',
        'httpx==0.25.2',
        'tiktoken==0.8.0',
        'pyyaml~=6.0.2',
        'protobuf~=5.29.2',
        'cryptography~=43.0.1'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)




import time

from cryptography.hazmat.primitives.asymmetric import rsa, padding
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.exceptions import InvalidKey, UnsupportedAlgorithm
from cryptography.hazmat.backends import default_backend


class RSACipher:
    HASH_ALGORITHM = hashes.SHA512()
    HASH_LENGTH = 64
    PADDING_OVERHEAD = 2 * HASH_LENGTH + 2
    KEY_SIZE = 4096
    MAX_ENCRYPT_LENGTH = (KEY_SIZE // 8) - PADDING_OVERHEAD

    def __init__(self, private_key=None, public_key=None):
        if private_key:
            self.private_key = private_key
        if public_key:
            self.public_key = public_key
        if not private_key and not public_key:
            self.private_key, self.public_key = self.generate_key_pair()

    def generate_key_pair(self):
        private_key = rsa.generate_private_key(
            public_exponent=65537,
            key_size=self.KEY_SIZE,
        )
        public_key = private_key.public_key()
        return private_key, public_key

    def encrypt(self, plaintext: bytes) -> bytes:
        try:
            if len(plaintext) > self.MAX_ENCRYPT_LENGTH:
                raise ValueError(
                    f"Plaintext too long. Max length is {self.MAX_ENCRYPT_LENGTH} bytes."
                )

            return self.public_key.encrypt(
                plaintext,
                padding.OAEP(
                    mgf=padding.MGF1(algorithm=self.HASH_ALGORITHM),
                    algorithm=self.HASH_ALGORITHM,
                    label=None,
                ),
            )
        except ValueError as e:
            raise ValueError(f"Encryption failed: {str(e)}")
        except InvalidKey:
            raise ValueError("Encryption failed: Invalid public key.")
        except UnsupportedAlgorithm:
            raise ValueError("Encryption failed: Unsupported algorithm.")
        except Exception as e:
            raise ValueError(f"Encryption failed: {str(e)}")

    def decrypt(self, ciphertext: bytes) -> bytes:
        try:
            return self.private_key.decrypt(
                ciphertext,
                padding.OAEP(
                    mgf=padding.MGF1(algorithm=self.HASH_ALGORITHM),
                    algorithm=self.HASH_ALGORITHM,
                    label=None,
                ),
            )
        except ValueError as e:
            raise ValueError(f"Decryption failed: {str(e)}")
        except InvalidKey:
            raise ValueError("Decryption failed: Invalid private key.")
        except UnsupportedAlgorithm:
            raise ValueError("Decryption failed: Unsupported algorithm.")
        except Exception as e:
            raise ValueError(f"Decryption failed: {str(e)}")

    def serialize_private_key(self) -> bytes:
        return self.private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption(),
        )

    def serialize_public_key(self) -> bytes:
        return self.public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo,
        )

    @staticmethod
    def deserialize_private_key(private_key_pem: bytes):
        try:
            return serialization.load_pem_private_key(
                private_key_pem,
                password=None,
                backend=default_backend()
            )
        except ValueError as e:
            print(f"Error deserializing private key: {e}")
            raise
        except Exception as e:
            print(f"Unexpected error deserializing private key: {e}")
            raise

    @staticmethod
    def deserialize_public_key(public_key_pem: bytes):
        try:
            return serialization.load_pem_public_key(
                public_key_pem,
                backend=default_backend()
            )
        except ValueError as e:
            print(f"Error deserializing public key: {e}")
            raise
        except Exception as e:
            print(f"Unexpected error deserializing public key: {e}")
            raise




from hwhkit.llm.config import load_models_from_yaml, load_models_from_dict

class LLMClient:
    _instance = None

    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self, config_file="llm_config.yaml", keys_file="llm_keys.yaml"):
        if not hasattr(self, 'models'):
            self.config_file = config_file
            self.keys_file = keys_file
            self.models = load_models_from_yaml(config_file=self.config_file, keys_file=self.keys_file)

    def __int__(self, config_dict: dict):
        if not hasattr(self, 'models'):
            self.models = load_models_from_dict(config_dict=config_dict)

    def list_models(self):
        if self.models is None:
            raise ValueError("Models failed to load.")
        return self.models.list_models()

    async def chat(self, model_name, prompt, system_prompt=""):
        if self.models is None:
            raise ValueError("Models failed to load.")
        return await self.models.get_model_instance(model_name).chat(prompt, system_prompt=system_prompt)

    async def chat_stream(self, model_name, prompt, system_prompt=""):
        if self.models is None:
            raise ValueError("Models failed to load.")
        async for chunk in self.models.get_model_instance(model_name).chat_stream(prompt, system_prompt=system_prompt):
            yield chunk

